# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['supervisor',
 'thoughtful',
 'thoughtful.supervisor',
 'thoughtful.supervisor.reporting']

package_data = \
{'': ['*']}

install_requires = \
['aws-requests-auth>=0.4.3,<0.5.0',
 'boto3>=1.24.64,<2.0.0',
 'chevron>=0.14.0,<0.15.0',
 'isodate>=0.6.1,<0.7.0',
 'moto>=4.0.5,<5.0.0',
 'pre-commit>=2.17.0,<3.0.0',
 'pyconfs>=0.5.5,<0.6.0',
 'pydantic-yaml>=0.6.3,<0.7.0',
 'pydantic>=1.8.2,<2.0.0',
 'pyyaml>=5.4.1']

setup_kwargs = {
    'name': 'thoughtful',
    'version': '1.16.0',
    'description': 'Thoughtful is a python package by Thoughtful for helping manage automations with helpful packages like supervisor',
    'long_description': '**thoughtful** is a collection of open-source libraries and tools for Robot Process\nAutomation (RPA) development. The goal of this project is to provide a set of\nfor supervising bot execution, and enabling these bots to do more.\n\n[![PyPi version](https://badgen.net/pypi/v/thoughtful/)](https://pypi.com/project/thoughtful)\n![Main Branch Tests](https://github.com/Thoughtful-Automation/supervisor/actions/workflows/main-push.yml/badge.svg?branch=main)\n[![Supported Versions](https://img.shields.io/pypi/pyversions/thoughtful.svg)](https://pypi.org/project/thoughtful)\n[![Downloads](https://pepy.tech/badge/thoughtful/month)](https://pepy.tech/project/thoughtful)\n\n[//]: # ([![GitHub release]&#40;https://img.shields.io/github/release/Thoughtful-Automation/supervisor.svg&#41;]&#40;https://GitHub.com/Naereen/StrapDown.js/releases/&#41;)\n\n\nThis project is:\n* Open-source: [GitHub][url:gh]\n* Owned by [thoughtful][url:ta]\n* Licensed under the [Apache License 2.0][url:al]\n\nLinks:\n* [Homepage][url:gh]\n* [Documentation][url:readthedocs]\n* [PyPI][url:pypi]\n\n**thoughtful** is available on [PyPI][url:pypi] and can be installed using pip:\n\n```sh\npip install thoughtful\n```\n\n---\n\n**thoughtful** officially supports Python 3.7+.\n\n---\n\n# Libraries\n\nThis is a list of the available libraries in this project. API Reference\nand User Guide available on [docs][url:readthedocs].\n\n## Supervisor\n\nSupervisor is a Workflow Engine for Digital Workers that constructs\nand broadcasts a detailed and structured telemetric log, called the Run Report.\n\n```python\nfrom thoughtful.supervisor import step, step_scope, supervise, set_step_status\n\n\n# using the step decorator\n@step("2")\ndef step_2(name: str) -> bool:\n    print(f\'Hello {name}\')\n    return True  # some condition\n\ndef main() -> None:\n    # using the step_scope context manager\n    with step_scope(\'1\') as step_context:\n        try:\n            print("Getting credentials")\n            # ...\n        except Exception as e:\n            # set step status using method\n            step_context.set_status("warning")\n\n    if not step_2():\n        # set step status using function\n        set_step_status("2", "fail")\n\nif __name__ == \'__main__\':\n    with supervise():\n        main()\n```\n\n## Contributing\n\nContributions to **thoughtful** are welcome!\n\nTo get started, see the [contributing guide](CONTRIBUTING.md).\n\n---\n\n  Made with ❤️ by\n\n  [![Thoughtful](https://user-images.githubusercontent.com/1096881/141985289-317c2e72-3c2d-4e6b-800a-0def1a05f599.png)][url:ta]\n\n---\n\nThis project is open-source and licensed under the terms of the [Apache License 2.0][url:al].\n\n\n<!--  Link References -->\n\n[url:ta]: https://www.thoughtful.ai/\n[url:gh]: https://github.com/Thoughtful-Automation/supervisor\n[url:pypi]: https://pypi.org/project/thoughtful/\n[git:issues]: https://github.com/Thoughtful-Automation/supervisor/issues\n[url:readthedocs]: https://thoughtful-supervisor.readthedocs-hosted.com/en/latest/\n[url:al]: http://www.apache.org/licenses/LICENSE-2.0\n',
    'author': 'Thoughtful Automation',
    'author_email': 'engineering@thoughtfulautomation.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://thoughtfulautomation.com',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



"""
This module contains the Recorder class. This class is responsible for
recording data and logs from the digital worker.

.. code-block:: python

    from thoughtful.supervisor import recorder, step, supervise


    @step("1.1")
    def do_something():
        try:
            ...  # do something
        except Exception as e:
            recorder.record_message(f"Something went wrong: {e}")


    def main():
        do_something()

        recorder.record_message("This is a message")


    if __name__ == "__main__":
        with supervise():
            main()

See more detailed examples below.
"""
from copy import deepcopy
from enum import Enum
from typing import Any, Dict, List, Optional, Tuple

MessageLog = List[str]
DataLog = List[Dict[str, Any]]


class Recorder:
    def __init__(self):
        """
        Logs messages, data, and changes to data during a ``DigitalWorker``
        execution.

        Attributes:
            self.data (list): Data logs.
            self.messages (list): A list of log messages.
        """

        self.data: DataLog = []
        self.messages: MessageLog = []

    def copy_logs(self) -> Tuple[MessageLog, DataLog]:
        """
        Creates deep copies of the logs recorded by this instance.

        Returns:
            Tuple[MessageLog, DataLog]: The log deep copies.
        """
        messages_copy: MessageLog = deepcopy(self.messages)
        data_copy: DataLog = deepcopy(self.data)
        return messages_copy, data_copy

    def record_data(
        self, log_data: any, label: str, description: Optional[str] = None
    ) -> any:
        """
        Log a piece of data for a step that's running.

        Examples:
            .. highlight:: python
            .. code-block:: python

                recorder = Recorder()
                recorder.record_data_change("Ticket#123",
                                             label="Support Ticket",
                                             description="The support ticket being used")

            .. code-block:: json

                "workflow": [
                    {
                    "step_id": "1.2.3",
                    "data_log": {
                        "type": "data",
                        "data": "Ticket#123",
                        "title": "Support Ticket",
                        "description": "The support ticket being used"
                    }
                ]

        Args:
            log_data (any): The data object to record, such as a string or dict.
            label (str): The name for the data object.
            description (str, optional): The description for the data, if any.

        Returns:
            any: The `log_data`.
        """

        new_entry = {
            "type": RecordType.DATA,
            "data": log_data,
            "label": label,
            "description": description,
        }

        # Cache the record (flushed at end of step execution)
        self.data.append(new_entry)

        # Return the original data
        return log_data

    def record_data_change(
        self,
        before: any,
        after: any,
        label: str,
        description: Optional[str] = None,
    ) -> any:
        """
        Record a change in data.

        Args:
            before (any): The data before the transformation.
            after (any): The data after the transformation.
            label: (str): The name for the data log.
            description (str, optional): The description for the data, if any.

        Example:
            .. highlight:: python
            .. code-block:: python

                recorder = Recorder()
                me = "Dave Arel"
                recorder.record_data_change(me,
                                            me.upper(),
                                            label="Uppercased My Name",
                                            description="Capitalizes my full name")

            .. code-block:: json

                "workflow": [
                    {
                    "step_id": "1.2.3",
                    "data_log": {
                        "type": "transformed",
                        "before": "Dave Arel",
                        "after": "DAVE AREL",
                        "label": "Uppercased Name",
                        "description": "Capitalizes the full name"
                    }
                ]

        Returns:
            any: The data from `before`.
        """

        transformation = {
            "type": RecordType.TRANSFORMATION,
            "before": before,
            "after": after,
        }
        self.record_data(transformation, label=label, description=description)
        return before

    def record_message(self, message: str) -> None:
        """
        Record a simple message.

        Args:
            message (str): The message to record.

        Examples:
            .. highlight:: python
            .. code-block:: python

                recorder = Recorder()
                recorder.record_data_change("message")

            .. code-block:: json

                "workflow": [
                    {
                    "step_id": "1.2.3",
                    "message_log": ["message"]
                ]

        Returns:
            None
        """

        # Cache the record (flushed at end of step execution)
        self.messages.append(message)


class RecordType(str, Enum):
    """
    Enum for the different types of manually recorded data structures. **This is
    not to be confused with the records referred to by the record IDs in the
    steps and step contexts.** Those records use the regular ``Status`` enum and
    are not tied in with this enum.

    Data structures of these types must be serializable.

    Transformed data structures can be a string, number, object or list
    of the former, as long as the data is serializable. This could be as
    simple as a string being capitalized, trimmed, edited, etc., or it
    can be as complex as a large tabular data set cleaned up to
    remove duplicates.

    This should be used as a way to record a transformation of data within a
    step so that the data can later be consumed in different ways, such as a
    visual-diff data coming out, or debugging to diagnose whether the problem
    was the data going in or the data going out.
    """

    DATA = "data"
    """A record that is simply recording a static data value"""

    TRANSFORMATION = "transformation"
    """A record that is a step_outputs of data being transformed"""
